@@ -34,6 +34,9 @@ static u64 load_4(const unsigned char *in) {
     return result;
 }
 
+static inline i64 shift_left(i64 v, int s) {
+	return i64(u64(v) << s);
+}
 
 
 /*
@@ -326,34 +329,34 @@ void fe_frombytes(fe h, const unsigned char *s) {
 
     carry9 = (h9 + (i64) (1 << 24)) >> 25;
     h0 += carry9 * 19;
-    h9 -= carry9 << 25;
+    h9 -= shift_left(carry9, 25);
     carry1 = (h1 + (i64) (1 << 24)) >> 25;
     h2 += carry1;
-    h1 -= carry1 << 25;
+    h1 -= shift_left(carry1, 25);
     carry3 = (h3 + (i64) (1 << 24)) >> 25;
     h4 += carry3;
-    h3 -= carry3 << 25;
+    h3 -= shift_left(carry3, 25);
     carry5 = (h5 + (i64) (1 << 24)) >> 25;
     h6 += carry5;
-    h5 -= carry5 << 25;
+    h5 -= shift_left(carry5, 25);
     carry7 = (h7 + (i64) (1 << 24)) >> 25;
     h8 += carry7;
-    h7 -= carry7 << 25;
+    h7 -= shift_left(carry7, 25);
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     carry2 = (h2 + (i64) (1 << 25)) >> 26;
     h3 += carry2;
-    h2 -= carry2 << 26;
+    h2 -= shift_left(carry2, 26);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry6 = (h6 + (i64) (1 << 25)) >> 26;
     h7 += carry6;
-    h6 -= carry6 << 26;
+    h6 -= shift_left(carry6, 26);
     carry8 = (h8 + (i64) (1 << 25)) >> 26;
     h9 += carry8;
-    h8 -= carry8 << 26;
+    h8 -= shift_left(carry8, 26);
 
     h[0] = (i32) h0;
     h[1] = (i32) h1;
@@ -528,7 +531,6 @@ int fe_isnonzero(const fe f) {
 }
 
 
-
 /*
     h = f * g
     Can overlap h with f or g.
@@ -719,46 +721,46 @@ void fe_mul(fe h, const fe f, const fe g) {
 
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
 
     carry1 = (h1 + (i64) (1 << 24)) >> 25;
     h2 += carry1;
-    h1 -= carry1 << 25;
+    h1 -= shift_left(carry1, 25);
     carry5 = (h5 + (i64) (1 << 24)) >> 25;
     h6 += carry5;
-    h5 -= carry5 << 25;
+    h5 -= shift_left(carry5, 25);
 
     carry2 = (h2 + (i64) (1 << 25)) >> 26;
     h3 += carry2;
-    h2 -= carry2 << 26;
+    h2 -= shift_left(carry2, 26);
     carry6 = (h6 + (i64) (1 << 25)) >> 26;
     h7 += carry6;
-    h6 -= carry6 << 26;
+    h6 -= shift_left(carry6, 26);
 
     carry3 = (h3 + (i64) (1 << 24)) >> 25;
     h4 += carry3;
-    h3 -= carry3 << 25;
+    h3 -= shift_left(carry3, 25);
     carry7 = (h7 + (i64) (1 << 24)) >> 25;
     h8 += carry7;
-    h7 -= carry7 << 25;
+    h7 -= shift_left(carry7, 25);
 
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry8 = (h8 + (i64) (1 << 25)) >> 26;
     h9 += carry8;
-    h8 -= carry8 << 26;
+    h8 -= shift_left(carry8, 26);
 
     carry9 = (h9 + (i64) (1 << 24)) >> 25;
     h0 += carry9 * 19;
-    h9 -= carry9 << 25;
+    h9 -= shift_left(carry9, 25);
 
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
 
     h[0] = (i32) h0;
     h[1] = (i32) h1;
@@ -816,17 +818,17 @@ void fe_mul121666(fe h, fe f) {
     i64 carry8;
     i64 carry9;
 
-    carry9 = (h9 + (i64) (1<<24)) >> 25; h0 += carry9 * 19; h9 -= carry9 << 25;
-    carry1 = (h1 + (i64) (1<<24)) >> 25; h2 += carry1; h1 -= carry1 << 25;
-    carry3 = (h3 + (i64) (1<<24)) >> 25; h4 += carry3; h3 -= carry3 << 25;
-    carry5 = (h5 + (i64) (1<<24)) >> 25; h6 += carry5; h5 -= carry5 << 25;
-    carry7 = (h7 + (i64) (1<<24)) >> 25; h8 += carry7; h7 -= carry7 << 25;
+    carry9 = (h9 + (i64) (1<<24)) >> 25; h0 += carry9 * 19; h9 -= shift_left(carry9, 25);
+    carry1 = (h1 + (i64) (1<<24)) >> 25; h2 += carry1; h1 -= shift_left(carry1, 25);
+    carry3 = (h3 + (i64) (1<<24)) >> 25; h4 += carry3; h3 -= shift_left(carry3, 25);
+    carry5 = (h5 + (i64) (1<<24)) >> 25; h6 += carry5; h5 -= shift_left(carry5, 25);
+    carry7 = (h7 + (i64) (1<<24)) >> 25; h8 += carry7; h7 -= shift_left(carry7, 25);
 
-    carry0 = (h0 + (i64) (1<<25)) >> 26; h1 += carry0; h0 -= carry0 << 26;
-    carry2 = (h2 + (i64) (1<<25)) >> 26; h3 += carry2; h2 -= carry2 << 26;
-    carry4 = (h4 + (i64) (1<<25)) >> 26; h5 += carry4; h4 -= carry4 << 26;
-    carry6 = (h6 + (i64) (1<<25)) >> 26; h7 += carry6; h6 -= carry6 << 26;
-    carry8 = (h8 + (i64) (1<<25)) >> 26; h9 += carry8; h8 -= carry8 << 26;
+    carry0 = (h0 + (i64) (1<<25)) >> 26; h1 += carry0; h0 -= shift_left(carry0, 26);
+    carry2 = (h2 + (i64) (1<<25)) >> 26; h3 += carry2; h2 -= shift_left(carry2, 26);
+    carry4 = (h4 + (i64) (1<<25)) >> 26; h5 += carry4; h4 -= shift_left(carry4, 26);
+    carry6 = (h6 + (i64) (1<<25)) >> 26; h7 += carry6; h6 -= shift_left(carry6, 26);
+    carry8 = (h8 + (i64) (1<<25)) >> 26; h9 += carry8; h8 -= shift_left(carry8, 26);
 
     h[0] = h0;
     h[1] = h1;
@@ -1088,40 +1090,40 @@ void fe_sq(fe h, const fe f) {
     i64 carry9;
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry1 = (h1 + (i64) (1 << 24)) >> 25;
     h2 += carry1;
-    h1 -= carry1 << 25;
+    h1 -= shift_left(carry1, 25);
     carry5 = (h5 + (i64) (1 << 24)) >> 25;
     h6 += carry5;
-    h5 -= carry5 << 25;
+    h5 -= shift_left(carry5, 25);
     carry2 = (h2 + (i64) (1 << 25)) >> 26;
     h3 += carry2;
-    h2 -= carry2 << 26;
+    h2 -= shift_left(carry2, 26);
     carry6 = (h6 + (i64) (1 << 25)) >> 26;
     h7 += carry6;
-    h6 -= carry6 << 26;
+    h6 -= shift_left(carry6, 26);
     carry3 = (h3 + (i64) (1 << 24)) >> 25;
     h4 += carry3;
-    h3 -= carry3 << 25;
+    h3 -= shift_left(carry3, 25);
     carry7 = (h7 + (i64) (1 << 24)) >> 25;
     h8 += carry7;
-    h7 -= carry7 << 25;
+    h7 -= shift_left(carry7, 25);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry8 = (h8 + (i64) (1 << 25)) >> 26;
     h9 += carry8;
-    h8 -= carry8 << 26;
+    h8 -= shift_left(carry8, 26);
     carry9 = (h9 + (i64) (1 << 24)) >> 25;
     h0 += carry9 * 19;
-    h9 -= carry9 << 25;
+    h9 -= shift_left(carry9, 25);
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     h[0] = (i32) h0;
     h[1] = (i32) h1;
     h[2] = (i32) h2;
@@ -1261,40 +1263,40 @@ void fe_sq2(fe h, const fe f) {
     h9 += h9;
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry1 = (h1 + (i64) (1 << 24)) >> 25;
     h2 += carry1;
-    h1 -= carry1 << 25;
+    h1 -= shift_left(carry1, 25);
     carry5 = (h5 + (i64) (1 << 24)) >> 25;
     h6 += carry5;
-    h5 -= carry5 << 25;
+    h5 -= shift_left(carry5, 25);
     carry2 = (h2 + (i64) (1 << 25)) >> 26;
     h3 += carry2;
-    h2 -= carry2 << 26;
+    h2 -= shift_left(carry2, 26);
     carry6 = (h6 + (i64) (1 << 25)) >> 26;
     h7 += carry6;
-    h6 -= carry6 << 26;
+    h6 -= shift_left(carry6, 26);
     carry3 = (h3 + (i64) (1 << 24)) >> 25;
     h4 += carry3;
-    h3 -= carry3 << 25;
+    h3 -= shift_left(carry3, 25);
     carry7 = (h7 + (i64) (1 << 24)) >> 25;
     h8 += carry7;
-    h7 -= carry7 << 25;
+    h7 -= shift_left(carry7, 25);
     carry4 = (h4 + (i64) (1 << 25)) >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry8 = (h8 + (i64) (1 << 25)) >> 26;
     h9 += carry8;
-    h8 -= carry8 << 26;
+    h8 -= shift_left(carry8, 26);
     carry9 = (h9 + (i64) (1 << 24)) >> 25;
     h0 += carry9 * 19;
-    h9 -= carry9 << 25;
+    h9 -= shift_left(carry9, 25);
     carry0 = (h0 + (i64) (1 << 25)) >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     h[0] = (i32) h0;
     h[1] = (i32) h1;
     h[2] = (i32) h2;
@@ -1429,33 +1431,33 @@ void fe_tobytes(unsigned char *s, const fe h) {
     /* Goal: Output h-2^255 q, which is between 0 and 2^255-20. */
     carry0 = h0 >> 26;
     h1 += carry0;
-    h0 -= carry0 << 26;
+    h0 -= shift_left(carry0, 26);
     carry1 = h1 >> 25;
     h2 += carry1;
-    h1 -= carry1 << 25;
+    h1 -= shift_left(carry1, 25);
     carry2 = h2 >> 26;
     h3 += carry2;
-    h2 -= carry2 << 26;
+    h2 -= shift_left(carry2, 26);
     carry3 = h3 >> 25;
     h4 += carry3;
-    h3 -= carry3 << 25;
+    h3 -= shift_left(carry3, 25);
     carry4 = h4 >> 26;
     h5 += carry4;
-    h4 -= carry4 << 26;
+    h4 -= shift_left(carry4, 26);
     carry5 = h5 >> 25;
     h6 += carry5;
-    h5 -= carry5 << 25;
+    h5 -= shift_left(carry5, 25);
     carry6 = h6 >> 26;
     h7 += carry6;
-    h6 -= carry6 << 26;
+    h6 -= shift_left(carry6, 26);
     carry7 = h7 >> 25;
     h8 += carry7;
-    h7 -= carry7 << 25;
+    h7 -= shift_left(carry7, 25);
     carry8 = h8 >> 26;
     h9 += carry8;
-    h8 -= carry8 << 26;
+    h8 -= shift_left(carry8, 26);
     carry9 = h9 >> 25;
-    h9 -= carry9 << 25;
+    h9 -= shift_left(carry9, 25);
 
     /* h10 = carry9 */
     /*
