@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2003-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -81,18 +81,14 @@ namespace libtorrent
 	// in case the function fails. ``error_pos`` is an optional pointer to an int,
 	// which will be set to the byte offset into the buffer where an error occurred,
 	// in case the function fails.
-	TORRENT_DEPRECATED
-	TORRENT_EXPORT int lazy_bdecode(char const* start, char const* end
+	TORRENT_DEPRECATED_EXPORT int lazy_bdecode(char const* start, char const* end
 		, lazy_entry& ret, error_code& ec, int* error_pos = 0
 		, int depth_limit = 1000, int item_limit = 1000000);
 
-#ifndef TORRENT_NO_DEPRECATE
 	// for backwards compatibility, does not report error code
 	// deprecated in 0.16
-	TORRENT_DEPRECATED
-	TORRENT_EXPORT int lazy_bdecode(char const* start, char const* end
+	TORRENT_DEPRECATED_EXPORT int lazy_bdecode(char const* start, char const* end
 		, lazy_entry& ret, int depth_limit = 1000, int item_limit = 1000000);
-#endif
 
 	// this is a string that is not NULL-terminated. Instead it
 	// comes with a length, specified in bytes. This is particularly
@@ -101,7 +97,7 @@ namespace libtorrent
 	// would require copying the string.
 	//
 	// see lazy_entry::string_pstr().
-	struct TORRENT_EXPORT pascal_string
+	struct TORRENT_DEPRECATED_EXPORT pascal_string
 	{
 		// construct a string pointing to the characters at ``p``
 		// of length ``l`` characters. No NULL termination is required.
@@ -136,7 +132,7 @@ namespace libtorrent
 	//
 	// There is also a ``none`` type, which is used for uninitialized
 	// lazy_entries.
-	struct TORRENT_EXPORT lazy_entry
+	struct TORRENT_DEPRECATED_EXPORT lazy_entry
 	{
 		// The different types a lazy_entry can have
 		enum entry_type_t
@@ -389,7 +385,7 @@ namespace libtorrent
 		lazy_entry const& operator=(lazy_entry const&);
 	};
 
-	struct lazy_dict_entry
+	struct TORRENT_DEPRECATED lazy_dict_entry
 	{
 		char const* name;
 		lazy_entry val;
@@ -397,11 +393,11 @@ namespace libtorrent
 
 	// print the bencoded structure in a human-readable format to a string
 	// that's returned.
-	TORRENT_DEPRECATED
-	TORRENT_EXPORT std::string print_entry(lazy_entry const& e
+	TORRENT_DEPRECATED_EXPORT std::string print_entry(lazy_entry const& e
 		, bool single_line = false, int indent = 0);
 
 	// defined in bdecode.cpp
+	TORRENT_DEPRECATED
 	TORRENT_EXTRA_EXPORT char const* parse_int(char const* start
 		, char const* end, char delimiter, boost::int64_t& val
 		, bdecode_errors::error_code_enum& ec);
