@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2011-2016, Arvid Norberg
+Copyright (c) 2011-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 namespace libtorrent
 {
-	typedef boost::uint32_t peer_class_t;
+	typedef int peer_class_t;
 
-	struct peer_class_info
+	// holds settings for a peer class. Used in set_peer_class() and
+	// get_peer_class() calls.
+	struct TORRENT_EXPORT peer_class_info
 	{
 		// ``ignore_unchoke_slots`` determines whether peers should always
 		// unchoke a peer, regardless of the choking algorithm, or if it should
