@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2003-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/weak_ptr.hpp>
 #include <boost/tuple/tuple.hpp>
 #include <boost/unordered_map.hpp>
+#include <boost/optional.hpp>
+#include <boost/noncopyable.hpp>
 
 #ifdef TORRENT_USE_OPENSSL
 // there is no forward declaration header for asio
@@ -164,7 +166,7 @@ namespace libtorrent
 #endif
 		sha1_hash info_hash;
 		peer_id pid;
-		address bind_ip;
+		boost::optional<address> bind_ip;
 
 		bool send_stats;
 
@@ -193,7 +195,7 @@ namespace libtorrent
 	{
 		tracker_response()
 			: interval(1800)
-			, min_interval(120)
+			, min_interval(1)
 			, complete(-1)
 			, incomplete(-1)
 			, downloaders(-1)
@@ -329,7 +331,6 @@ namespace libtorrent
 			, int interval = 0, int min_interval = 0);
 		virtual void start() = 0;
 		virtual void close();
-		address const& bind_interface() const { return m_req.bind_ip; }
 		void sent_bytes(int bytes);
 		void received_bytes(int bytes);
 		virtual bool on_receive(error_code const&, udp::endpoint const&
