@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007-2016, Arvid Norberg
+Copyright (c) 2007-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -1159,7 +1159,7 @@ struct upnp_error_category : boost::system::error_category
 {
 	virtual const char* name() const BOOST_SYSTEM_NOEXCEPT
 	{
-		return "UPnP error";
+		return "upnp";
 	}
 
 	virtual std::string message(int ev) const BOOST_SYSTEM_NOEXCEPT
